EMPTY  = 0
BLUE   = 1
RED    = 2
LIKEME = 4

function wait(n) 
	for i = 0, n*100000 do io.write("") end -- do nothing 100.000 times
end

function createCellSpace (model)
	model.cellspace = CellularSpace {
		xdim = model.cellSpaceDim,
		ydim = model.cellSpaceDim
	}

    model.cellspace:createNeighborhood {
        strategy = "moore",
  		self = false
	}

	forEachCell (model.cellspace, function (cell)
		cell.color = EMPTY
	end)

end

function satisfied (agent)

	num_equal = 0 
	cell = agent:getCell()

	forEachNeighbor (cell, function (cell, neigh)
		if neigh.color == cell.color then
			num_equal = num_equal + 1
		end
	end)

	if (num_equal >= LIKEME) then
		return true
	else
		return false
	end
end

function moveFamily (family)

	--- try to get a place to go 
	newcell = model.cellspace:sample()

	-- is the new cell empty? Yes, move; No, stay
	if newcell.color == EMPTY then
		family:getCell().color = EMPTY
		family:move (newcell)
		family:getCell().color = family.state
	end
end


function createAgent (model)

	model.family = Agent {
	-- an Agent has two functions (init and execute)
	-- init initializes the agent
	-- execute runs the agent
		-- initialize the agent

		init = function (self)
			if math.random() < 0.5 then
				self.state = BLUE
			else
				self.state = RED
			end
			-- self.state = math.random(2)
		end,

		execute = function (self)
			if not satisfied (self) then
				moveFamily (self)
			end
		end
	}
end

function createSociety (model)
	model.society = Society {
		instance = model.family, 
		quantity = model.societyDim
	}
end

function createEnvironment (model)

	model.env = Environment {model.cellspace, model.society}

	model.env:createPlacement {
		strategy = "random",
		max = 1 
	}

	forEachAgent (model.society, function (agent)
		agent:getCell().color = agent.state
		end)

end

function createTimer (model)
	model.timer = Timer {
		Event {time = 1, period = 1, action = model.society},
		Event {time = 1, period = 1, action = model.cellspace},
		Event {time = 1, period = 1, action = wait(50)}
	}
end 

function createMap (cellspace)

	leg = Legend {
		grouping = "uniquevalue",
		colorBar = {
			{value = EMPTY, color = "white"},
			{value = BLUE,  color = "blue"},
			{value = RED,   color = "red"}
		}
	}
	map = Observer {
		type = "map",
		subject = cellspace,
		attributes = {"color"},
		legends = {leg}
	}
end

function AgentModel (model)

	-- createCellSpace 
	createCellSpace (model)  -- Ok

	-- create Agent
	createAgent (model)   -- OK

	-- create Society
	createSociety (model) --oK

	-- create Environment 
	createEnvironment (model) -- OK

	-- timer 
	createTimer (model) -- OK

	-- create Observer
	createMap (model.cellspace) -- OK

	return model
end

function run (model, time)
	model.timer:execute (time)
end

model = AgentModel {
	cellSpaceDim  =  50,
	societyDim    =  2400,
}

run (model, 300)